@@ -99,9 +99,23 @@
 #include "qmi-enums-fox.h"
 #include "qmi-fox.h"
 
+#include "qmi-atr.h"
+
+#include "qmi-enums-imsp.h"
+#include "qmi-imsp.h"
+
+#include "qmi-enums-imsa.h"
+#include "qmi-imsa.h"
+
+#include "qmi-ims.h"
+
+#include "qmi-enums-ssc.h"
+#include "qmi-ssc.h"
+
 /* generated */
 #include "qmi-error-types.h"
 #include "qmi-enum-types.h"
+#include "qmi-flag-types.h"
 #include "qmi-flags64-types.h"
 
 #if QMI_QRTR_SUPPORTED
