@@ -42,17 +42,6 @@
 
 G_BEGIN_DECLS
 
-/**
- * SECTION:qmi-device
- * @title: QmiDevice
- * @short_description: Generic QMI device handling routines
- *
- * #QmiDevice is a generic type in charge of controlling the access of multiple
- * #QmiClient objects to the managed QMI port.
- *
- * A #QmiDevice can only handle one single QMI port.
- */
-
 #define QMI_TYPE_DEVICE            (qmi_device_get_type ())
 #define QMI_DEVICE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), QMI_TYPE_DEVICE, QmiDevice))
 #define QMI_DEVICE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  QMI_TYPE_DEVICE, QmiDeviceClass))
@@ -100,6 +89,15 @@ typedef struct _QmiDevicePrivate QmiDevicePrivate;
  */
 #define QMI_DEVICE_WWAN_IFACE "device-wwan-iface"
 
+/**
+ * QMI_DEVICE_CONSECUTIVE_TIMEOUTS:
+ *
+ * Symbol defining the #QmiDevice:device-consecutive-timeouts property.
+ *
+ * Since: 1.32
+ */
+#define QMI_DEVICE_CONSECUTIVE_TIMEOUTS "device-consecutive-timeouts"
+
 /**
  * QMI_DEVICE_SIGNAL_INDICATION:
  *
@@ -121,6 +119,13 @@ typedef struct _QmiDevicePrivate QmiDevicePrivate;
 /**
  * QmiDevice:
  *
+ * Generic QMI device handling routines
+ *
+ * `QmiDevice` is a generic type in charge of controlling the access of multiple
+ * [class@Client] objects to the managed QMI port.
+ *
+ * A `QmiDevice` can only handle one single QMI port.
+
  * The #QmiDevice structure contains private data and should only be accessed
  * using the provided API.
  *
@@ -238,13 +243,13 @@ gboolean qmi_device_is_open (QmiDevice *self);
  * @QMI_DEVICE_OPEN_FLAGS_NONE: No flags.
  * @QMI_DEVICE_OPEN_FLAGS_VERSION_INFO: Run version info check when opening. Since 1.24.4 this flag no longer prevents requests from being sent to the modem if they're assumed not supported based on the version info of each message.
  * @QMI_DEVICE_OPEN_FLAGS_SYNC: Synchronize with endpoint once the device is open. Will release any previously allocated client ID.
- * @QMI_DEVICE_OPEN_FLAGS_NET_802_3: set network port to "802.3" mode; mutually exclusive with @QMI_DEVICE_OPEN_FLAGS_NET_RAW_IP
- * @QMI_DEVICE_OPEN_FLAGS_NET_RAW_IP: set network port to "raw IP" mode; mutally exclusive with @QMI_DEVICE_OPEN_FLAGS_NET_802_3
- * @QMI_DEVICE_OPEN_FLAGS_NET_QOS_HEADER: set network port to transmit/receive QoS headers; mutually exclusive with @QMI_DEVICE_OPEN_FLAGS_NET_NO_QOS_HEADER
- * @QMI_DEVICE_OPEN_FLAGS_NET_NO_QOS_HEADER: set network port to not transmit/receive QoS headers; mutually exclusive with @QMI_DEVICE_OPEN_FLAGS_NET_QOS_HEADER
- * @QMI_DEVICE_OPEN_FLAGS_PROXY: Try to open the port through the 'qmi-proxy'. Since: 1.8.
- * @QMI_DEVICE_OPEN_FLAGS_MBIM: open an MBIM port with QMUX tunneling service. Since: 1.16.
- * @QMI_DEVICE_OPEN_FLAGS_AUTO: open a port either in QMI or MBIM mode, depending on device driver. Since: 1.18.
+ * @QMI_DEVICE_OPEN_FLAGS_NET_802_3: Set network port to "802.3" mode; mutually exclusive with @QMI_DEVICE_OPEN_FLAGS_NET_RAW_IP
+ * @QMI_DEVICE_OPEN_FLAGS_NET_RAW_IP: Set network port to "raw IP" mode; mutally exclusive with @QMI_DEVICE_OPEN_FLAGS_NET_802_3
+ * @QMI_DEVICE_OPEN_FLAGS_NET_QOS_HEADER: Set network port to transmit/receive QoS headers; mutually exclusive with @QMI_DEVICE_OPEN_FLAGS_NET_NO_QOS_HEADER
+ * @QMI_DEVICE_OPEN_FLAGS_NET_NO_QOS_HEADER: Set network port to not transmit/receive QoS headers; mutually exclusive with @QMI_DEVICE_OPEN_FLAGS_NET_QOS_HEADER
+ * @QMI_DEVICE_OPEN_FLAGS_PROXY: Try to open the port through the 'qmi-proxy'. Since 1.8.
+ * @QMI_DEVICE_OPEN_FLAGS_MBIM: Open an MBIM port with QMUX tunneling service. Since 1.16.
+ * @QMI_DEVICE_OPEN_FLAGS_AUTO: Open a port either in QMI or MBIM mode, depending on device driver. Since 1.18.
  * @QMI_DEVICE_OPEN_FLAGS_EXPECT_INDICATIONS: Explicitly state that indications are wanted (implicit in QMI mode, optional when in MBIM mode).
  *
  * Flags to specify which actions to be performed when the device is open.
@@ -351,7 +356,7 @@ gboolean qmi_device_close_finish (QmiDevice     *self,
  * qmi_device_allocate_client:
  * @self: a #QmiDevice.
  * @service: a valid #QmiService.
- * @cid: a valid client ID, or #QMI_CID_NONE.
+ * @cid: a valid client ID, or %QMI_CID_NONE.
  * @timeout: maximum time to wait.
  * @cancellable: optional #GCancellable object, %NULL to ignore.
  * @callback: a #GAsyncReadyCallback to call when the operation is finished.
@@ -359,13 +364,13 @@ gboolean qmi_device_close_finish (QmiDevice     *self,
  *
  * Asynchronously allocates a new #QmiClient in @self.
  *
- * If #QMI_CID_NONE is given in @cid, a new client ID will be allocated;
+ * If %QMI_CID_NONE is given in @cid, a new client ID will be allocated;
  * otherwise a client with the given @cid will be generated.
  *
  * When the operation is finished @callback will be called. You can then call
  * qmi_device_allocate_client_finish() to get the result of the operation.
  *
- * Note: Clients for the #QMI_SERVICE_CTL cannot be created with this method;
+ * Note: Clients for the %QMI_SERVICE_CTL cannot be created with this method;
  * instead get/peek the implicit one from @self.
  *
  * Since: 1.0
@@ -595,9 +600,9 @@ typedef gboolean (* QmiDeviceCommandAbortableParseResponseFn) (QmiDevice   *self
  * @message: the message to send.
  * @message_context: the context of the message.
  * @timeout: maximum time, in seconds, to wait for the response.
- * @abort_build_request_fn: (scope async): callback to build an abort request.
- * @abort_parse_response_fn: (scope async): callback to parse an abort response.
- * @abort_user_data: (closure): user data to pass to @build_request_fn and @parse_response_fn.
+ * @abort_build_request_fn: (scope async) (closure abort_user_data): callback to build an abort request.
+ * @abort_parse_response_fn: (scope async) (closure abort_user_data): callback to parse an abort response.
+ * @abort_user_data: user data to pass to @abort_build_request_fn and @abort_parse_response_fn.
  * @abort_user_data_free: (destroy abort_user_data): a #GDestroyNotify to free @abort_user_data.
  * @cancellable: a #GCancellable, or %NULL.
  * @callback: a #GAsyncReadyCallback to call when the operation is finished.
@@ -721,6 +726,18 @@ GArray *qmi_device_get_service_version_info_finish (QmiDevice     *self,
                                                     GAsyncResult  *res,
                                                     GError       **error);
 
+/**
+ * qmi_device_get_consecutive_timeouts:
+ * @self: a #QmiDevice.
+ *
+ * Gets the number of consecutive transaction timeouts in the device.
+ *
+ * Returns: a #guint.
+ *
+ * Since: 1.32
+ */
+guint qmi_device_get_consecutive_timeouts (QmiDevice *self);
+
 /******************************************************************************/
 /* qmi_wwan specific APIs */
 
@@ -873,8 +890,8 @@ gboolean qmi_device_check_expected_data_format_supported (QmiDevice
  * qmi_device_add_link:
  * @self: a #QmiDevice.
  * @mux_id: the mux id for the link, in the
- *   [#QMI_DEVICE_MUX_ID_MIN,#QMI_DEVICE_MUX_ID_MAX] range, or
- *   #QMI_DEVICE_MUX_ID_AUTOMATIC to find the first available mux id.
+ *   [%QMI_DEVICE_MUX_ID_MIN,%QMI_DEVICE_MUX_ID_MAX] range, or
+ *   %QMI_DEVICE_MUX_ID_AUTOMATIC to find the first available mux id.
  * @base_ifname: the interface which the new link will be created on.
  * @ifname_prefix: the prefix suggested to be used for the name of the new link
  *   created.
@@ -915,7 +932,7 @@ gboolean qmi_device_check_expected_data_format_supported (QmiDevice
  *
  * <note><para>
  * When using the qmi_wwan driver from a kernel older than v5.12, some of the
- * multiplexing features like using #QMI_DEVICE_MUX_ID_AUTOMATIC may not be fully
+ * multiplexing features like using %QMI_DEVICE_MUX_ID_AUTOMATIC may not be fully
  * available for programs that use ephimeral #QmiDevice objects for single
  * operations.
  * </para></note>
@@ -948,6 +965,86 @@ gchar *qmi_device_add_link_finish (QmiDevice     *self,
                                    guint         *mux_id,
                                    GError       **error);
 
+/**
+ * QmiDeviceAddLinkFlags:
+ * @QMI_DEVICE_ADD_LINK_FLAGS_NONE: none.
+ * @QMI_DEVICE_ADD_LINK_FLAGS_INGRESS_MAP_CKSUMV4: checksum offload (v4) is enabled in ingress path.
+ * @QMI_DEVICE_ADD_LINK_FLAGS_EGRESS_MAP_CKSUMV4: checksum offload (v4) is enabled in egress path.
+ * @QMI_DEVICE_ADD_LINK_FLAGS_INGRESS_MAP_CKSUMV5: checksum offload (v5) is enabled in ingress path. Since 1.32.
+ * @QMI_DEVICE_ADD_LINK_FLAGS_EGRESS_MAP_CKSUMV5: checksum offload (v5) is enabled in egress path. Since 1.32.
+ *
+ * Flags specifying the behavior of the new link.
+ *
+ * Since: 1.30
+ */
+typedef enum { /*< since=1.30 >*/
+    QMI_DEVICE_ADD_LINK_FLAGS_NONE                = 0,
+    QMI_DEVICE_ADD_LINK_FLAGS_INGRESS_MAP_CKSUMV4 = 1 << 1,
+    QMI_DEVICE_ADD_LINK_FLAGS_EGRESS_MAP_CKSUMV4  = 1 << 2,
+    QMI_DEVICE_ADD_LINK_FLAGS_INGRESS_MAP_CKSUMV5 = 1 << 3,
+    QMI_DEVICE_ADD_LINK_FLAGS_EGRESS_MAP_CKSUMV5  = 1 << 4,
+} QmiDeviceAddLinkFlags;
+
+/**
+ * qmi_device_add_link_with_flags:
+ * @self: a #QmiDevice.
+ * @mux_id: the mux id for the link, in the
+ *   [%QMI_DEVICE_MUX_ID_MIN,%QMI_DEVICE_MUX_ID_MAX] range, or
+ *   %QMI_DEVICE_MUX_ID_AUTOMATIC to find the first available mux id.
+ * @base_ifname: the interface which the new link will be created on.
+ * @ifname_prefix: the prefix suggested to be used for the name of the new link
+ *   created.
+ * @flags: bitmask of %QmiDeviceAddLinkFlags values to pass to the kernel when
+ *   creating the new link.
+ * @cancellable: a #GCancellable, or %NULL.
+ * @callback: a #GAsyncReadyCallback to call when the operation is finished.
+ * @user_data: the data to pass to callback function.
+ *
+ * Asynchronously creates a new virtual network device in the same way as
+ * qmi_device_add_link() does, but passing the additional @flags to the kernel
+ * during the operation.
+ *
+ * Using %QMI_DEVICE_ADD_LINK_FLAGS_NONE as @flags is equivalent to calling
+ * qmi_device_add_link() directly.
+ *
+ * If the link creation with the given set of @flags is unsupported by the
+ * backend, the operation may fail.
+ *
+ * <note><para>
+ * None of the @flags supported are applicable when using the multiplexing
+ * support provided by the qmi_wwan kernel driver, they are only used if using
+ * the rmnet backend for link management support.
+ * </para></note>
+ *
+ * Since: 1.30
+ */
+void qmi_device_add_link_with_flags (QmiDevice             *self,
+                                     guint                  mux_id,
+                                     const gchar           *base_ifname,
+                                     const gchar           *ifname_prefix,
+                                     QmiDeviceAddLinkFlags  flags,
+                                     GCancellable          *cancellable,
+                                     GAsyncReadyCallback    callback,
+                                     gpointer               user_data);
+
+/**
+ * qmi_device_add_link_with_flags_finish:
+ * @self: a #QmiDevice.
+ * @res: a #GAsyncResult.
+ * @mux_id: the mux ID for the link created.
+ * @error: Return location for error or %NULL.
+ *
+ * Finishes an operation started with qmi_device_add_link_finish().
+ *
+ * Returns: The name of the net interface created, %NULL if @error is set.
+ *
+ * Since: 1.30
+ */
+gchar *qmi_device_add_link_with_flags_finish (QmiDevice     *self,
+                                              GAsyncResult  *res,
+                                              guint         *mux_id,
+                                              GError       **error);
+
 /**
  * qmi_device_delete_link:
  * @self: a #QmiDevice.
@@ -1051,6 +1148,9 @@ gboolean qmi_device_delete_all_links_finish (QmiDevice     *self,
  *   output #GPtrArray of link names.
  * @error: Return location for error or %NULL.
  *
+ * Synchronously lists all virtual network interfaces that have been previously
+ * created with qmi_device_add_link() in @base_ifname.
+ *
  * Returns: %TRUE if successful, %FALSE if @error is set.
  *
  * Since: 1.28
